
import java.util.*;

// The whole system. The highest command-level in the application.
class ttchess_controller extends ttchess_common {

	ttchess_database db; // a model of all boards
	ttchess_interpreter ipr; // interprets messages and alters the database
	ttchess_receiver rec; // splits incoming datastream into messages
	ttchess_serialport ser; // low-level serial port communication
	ttchess_sender snd; // formatting and sending of outgoing commands to the board(s)

	public ttchess_controller() {
		db = new ttchess_database();
		ipr = new ttchess_interpreter(db);
		rec = new ttchess_receiver(ipr);
		ser = new ttchess_serialport(rec);
		snd = new ttchess_sender(ser);
	}

	boolean connect(String comport) {
		if (ser.open(comport)==false) {
			System.out.println("Could not open "+comport);
			return false;
		}
		System.out.println("COM port opened: "+comport);
		return true;
	}

	// Try to find existing and new boards. Do not remove
	// bords from the database if a board does not answer.
	// Found boards will have their alive==true when this is done.
	int find_all_boards() {
		System.out.println("\nScanning for boards...");
		db.unalive_all();  // set alive=false on all boards in the database
		int alive = 0;
		while (true) {
			try {Thread.sleep(300);} catch(Throwable t) {}
			snd.send(DGT_BUS_PING, 0); // broadcast ping to address 0
			// the replys are randomly spread over a 1100 ms interval.
			try {Thread.sleep(1100);} catch(Throwable t) {}
			int alivenow = db.count_alive(); // count boards that have ping-replyed so far
			if (alivenow == alive) break; // quit if no new board replyed
			alive = alivenow;
			// make sure that already replyed boards don't reply to next ping
			for (Enumeration e = db.get_boards() ; e.hasMoreElements() ;) {
				ttchess_chessboard b = (ttchess_chessboard)(e.nextElement());
				if (b.get_alive()==true) {
					snd.send(DGT_BUS_IGNORE_NEXT_BUS_PING, b.get_address());
				}
			}
		}
		System.out.println("Scanning completed, found "+Integer.toString(alive)+" boards.\n");
		return alive;
	}

	// Gets a full dump from all active boards. If a board does not answer
	// within 3 calls it is assumed to be dead and will not be called again
	// until it has replyed to a broadcast ping.
	void dump_all_active_boards() {
		for (Enumeration e = db.get_boards() ; e.hasMoreElements() ;) {
			ttchess_chessboard b = (ttchess_chessboard)(e.nextElement());
			if (b.get_alive()==true) {
				b.set_dumped(false);
				for (int i=0; i<3; i=i+1) {  // 3 retries
					if (b.get_dumped()==true) break;  // ok, it has received a dump message
					snd.send(DGT_BUS_SEND_BRD, b.get_address());
					try {Thread.sleep(200);} catch(Throwable t) {} // My choice of time-out
				}
				// put the board offline if the board did not answer 
				if (b.get_dumped()==false) {
					System.out.println("Board "+Integer.toString(b.get_address())+" does not reply to dump_board.");
					b.set_alive(false);
				}
			}
		}
	}
	
	// Gets changes from all active boards. If a board does not answer
	// within 3 calls it is assumed to be dead and will not be called again
	// until it has replyed to a broadcast ping.
	void poll_all_active_boards() {
		int TIMEOUT = 100; // recommended
		//int TIMEOUT = 3000;
		for (Enumeration e = db.get_boards() ; e.hasMoreElements() ;) {
			ttchess_chessboard b = (ttchess_chessboard)(e.nextElement());
			if (b.get_alive()==true) {
				b.set_updated(false);
				snd.send(DGT_BUS_SEND_CHANGES, b.get_address());
				try {Thread.sleep(TIMEOUT);} catch(Throwable t) {} // Recommended time-out
				for (int i=0; i<2; i=i+1) {  // 2 retries
					if (b.get_updated()==true) break;  // ok
					snd.send(DGT_BUS_REPEAT_CHANGES, b.get_address());
					try {Thread.sleep(TIMEOUT);} catch(Throwable t) {} // Recommended time-out
				}
				// put the board offline if the board did not answer 
				if (b.get_updated()==false) {
					System.out.println("Board "+Integer.toString(b.get_address())+" does not reply anymore.");
					b.set_alive(false);
				}
			}
		}
	}
}
